@@ -1,15 +1,17 @@
-import dill as pickle
+import dill
 
 
 class StorageMixin:
+    """Mixin class for serialization."""
+
     def __init__(self, *args, **kwargs):
         super(StorageMixin, self).__init__(*args, **kwargs)
 
     def save(self, path: str) -> None:
         """Save the object."""
-        pickle.dump(self, open(path, "wb"))
+        dill.dump(self, open(path, "wb"))
 
     @classmethod
-    def load(cls, path: str):
+    def load(cls, path: str) -> object:
         """Load the object from the path."""
-        return pickle.load(open(path, "rb"))
+        return dill.load(open(path, "rb"))
