@@ -1,13 +1,21 @@
-from typing import Callable, Collection, Dict, OrderedDict
+from __future__ import annotations
+
+from collections import OrderedDict
+from typing import Callable, Collection, Dict, List, Tuple
 
 from datasets.features import FeatureType
 
+from robustnessgym.core.slice import SliceDataPanel as DataPanel
 from robustnessgym.core.tools import get_all_paths
 
 
 class Schema:
+    """Class for task input/output schemas in Robustness Gym."""
+
     def __init__(
-        self, features: OrderedDict, grounding_candidates: Dict[str, Collection]
+        self,
+        features: OrderedDict,
+        grounding_candidates: Dict[str, Collection],
     ):
         # Store the features and grounding candidates
         self.features = features
@@ -16,11 +24,42 @@ class Schema:
             v: k for k, values in self.grounding_candidates.items() for v in values
         }
 
+    @classmethod
+    def from_columns(
+        cls,
+        features: Dict[str, FeatureType],
+        columns: List[str],
+    ) -> Schema:
+        """Create a schema using features and columns."""
+        for col in columns:
+            assert col in features, f"Column {col} must be in `features`."
+
+        return Schema(
+            features=OrderedDict({k: v for k, v in features.items() if k in columns}),
+            grounding_candidates={k: {k} for k in columns},
+        )
+
+    @classmethod
+    def for_dataset(
+        cls,
+        dp: DataPanel,
+        input_columns: List[str],
+        output_columns: List[str],
+    ) -> Tuple[Schema, Schema]:
+        """Create input and output schemas using features, input and output
+        columns."""
+        # Set the features
+        features = dp.features
+
+        return Schema.from_columns(features, input_columns), Schema.from_columns(
+            features, output_columns
+        )
+
     def ground(self, features: Dict[str, FeatureType]):
         """
 
         Args:
-            features: given by Dataset.features
+            features: given by DataPanel.features
 
         Returns: (grounding, reversed_grounding)
 
@@ -32,7 +71,7 @@ class Schema:
             tuple(path) if len(path) > 1 else path[0] for path in flat_columns
         }
 
-        # Figure out the (reversed) grounding: map columns in the dataset to keys in
+        # Figure out the (reversed) grounding: map columns in the dp to keys in
         # the schema
         reversed_grounding = {}
         for k in self.reversed_grounding_candidates:
@@ -49,7 +88,7 @@ class Schema:
 
         # Assert that the grounded schema has the right types
         # FIXME(karan): Value == ClassLabel should be allowed: shouldn't break this
-        # TODO(karan): if not, add code to automatically rejig the dataset in map_fn
+        # TODO(karan): if not, add code to automatically rejig the dp in map_fn
         # for key in self.features:
         #     if isinstance(grounding[key], str):
         #         assert self.features[key] == features[grounding[key]]
@@ -65,5 +104,7 @@ class Schema:
     def __len__(self):
         return len(self.features)
 
-    def keys(self):
+    @property
+    def columns(self):
+        """List of columns that participate in the schema."""
         return list(self.features.keys())
